package com.tcctyn.iot.forestfire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctyn.common.core.exception.ServiceException;
import com.tcctyn.common.core.utils.DateUtils;
import com.tcctyn.iot.forestfire.domain.CheckInRecord;
import com.tcctyn.iot.forestfire.domain.PointInfo;
import com.tcctyn.iot.forestfire.domain.Schedules;
import com.tcctyn.iot.forestfire.enums.PointTypeEnum;
import com.tcctyn.iot.forestfire.mapper.CheckInRecordMapper;
import com.tcctyn.iot.forestfire.mapper.PointInfoMapper;
import com.tcctyn.iot.forestfire.mapper.SchedulesMapper;
import com.tcctyn.iot.forestfire.service.ICheckInRecordService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 打卡记录表 服务实现类
 * </p>
 *
 * @author liuzh
 * @since 2024-10-12
 */
@Service
public class CheckInRecordServiceImpl extends ServiceImpl<CheckInRecordMapper, CheckInRecord> implements ICheckInRecordService {


    @Resource
    private CheckInRecordMapper checkInRecordMapper;

    @Resource
    private SchedulesMapper schedulesMapper;

    @Resource
    private PointInfoMapper pointInfoMapper;

    @Override
    public List<CheckInRecord> list(CheckInRecord checkInRecord) {
        QueryWrapper<CheckInRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(checkInRecord);
        return checkInRecordMapper.selectList(queryWrapper.lambda()
                .eq(CheckInRecord::getEffectiveFlag, 1)
        );
    }

    @Override
    public CheckInRecord getById(Long id) {
        CheckInRecord checkInRecord = checkInRecordMapper.selectById(id);
        return checkInRecord;
    }

    @Override
    public Integer create(CheckInRecord checkInRecord, Long userId) {
        String today = DateUtils.getDate();
        Schedules schedules = schedulesMapper.selectOne(new QueryWrapper<Schedules>().lambda()
                .eq(Schedules::getUserId, userId)
                .eq(Schedules::getDateOfWeek, today)
        );
        if (Objects.isNull(schedules)) {
            throw new ServiceException("没有查到排班信息！请核查");
        }
        //在更新打卡前，需要查询是否该排班、该点位已经有有效打卡记录，如果有，则需要置为失效打卡记录；然后再插入新的打卡记录；
        //一个班次，一个打卡点，一个人只能有一条有效打卡记录（可能存在多个人巡一条路径的情况）
        CheckInRecord existCheckInRecord = checkInRecordMapper.selectOne(new QueryWrapper<CheckInRecord>().lambda()
                .eq(CheckInRecord::getScheduleId, schedules.getScheduleId())
                .eq(CheckInRecord::getPointId, checkInRecord.getPointId())
                .eq(CheckInRecord::getUserId, userId)
                .eq(CheckInRecord::getEffectiveFlag, 1)
        );
        if (Objects.nonNull(existCheckInRecord)) {
            existCheckInRecord.setEffectiveFlag(0);
            checkInRecordMapper.updateById(existCheckInRecord);
        } else {
            schedules.setCheckTimes(schedules.getCheckTimes() + 1);
        }
        //插入新的打卡记录
        checkInRecord.setScheduleId(schedules.getScheduleId());
        checkInRecord.setForestId(schedules.getForestId());
        checkInRecord.setFarmId(schedules.getFarmId());
        checkInRecord.setFenceId(schedules.getFenceId());
        checkInRecord.setPathId(schedules.getPathId());
        checkInRecord.setUserId(userId);
        checkInRecord.setCheckInTime(new Date());

        //插入新的打卡记录后，需要更新排班表中的打卡次数，应打卡次数、巡护进度
        List<PointInfo> pointInfoList = pointInfoMapper.selectList(new QueryWrapper<PointInfo>().lambda()
                .eq(PointInfo::getPointType, PointTypeEnum.CHECK_IN_POINT.name())
                .eq(PointInfo::getBelongingRegionId, schedules.getPathId())
        );
        Integer totalTimes = pointInfoList.size();
        Schedules schedulesUpdate = new Schedules();
        schedulesUpdate.setScheduleId(schedules.getScheduleId());
        schedulesUpdate.setTotalTimes(totalTimes);
        schedulesUpdate.setCheckTimes(schedules.getCheckTimes());
        String patrolRate = "0%";
        if (totalTimes > 0) {
            patrolRate = String.format("%.2f", (schedulesUpdate.getCheckTimes() * 100.0 / schedulesUpdate.getTotalTimes())) + "%";
        }
        schedulesUpdate.setPatrolRate(patrolRate);
        schedulesMapper.updateById(schedulesUpdate);
        return checkInRecordMapper.insert(checkInRecord);
    }

    @Override
    public Integer delete(Long id) {
        return checkInRecordMapper.deleteById(id);
    }

    @Override
    public Integer update(CheckInRecord checkInRecord) {
        return checkInRecordMapper.updateById(checkInRecord);
    }

    @Override
    public List<CheckInRecord> getByUserId(Long userId, Date checkDate) {
        QueryWrapper<CheckInRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_id", userId);
        queryWrapper.eq("effective_flag", 1);
        //指定日期转换为字符串
        queryWrapper.apply("date_format(check_in_time,'%Y%m%d')=" + "'" + DateUtils.parseDateToStr("yyyyMMdd", checkDate) + "'");

        return checkInRecordMapper.selectList(queryWrapper);
    }

    @Override
    public List<CheckInRecord> selectLast10Items() {
        List<CheckInRecord> checkInRecordList = checkInRecordMapper.selectList(new QueryWrapper<CheckInRecord>().lambda()
                .eq(CheckInRecord::getEffectiveFlag, 1)
                .orderByDesc(CheckInRecord::getCheckInTime)
                .last("limit 10")
        );
        return checkInRecordList;
    }


}
