package com.tcctyn.iot.forestfire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctyn.common.core.exception.ServiceException;
import com.tcctyn.common.core.utils.bean.BeanUtils;
import com.tcctyn.iot.forestfire.VO.*;
import com.tcctyn.iot.forestfire.domain.DeviceChanels;
import com.tcctyn.iot.forestfire.domain.DeviceInfo;
import com.tcctyn.iot.forestfire.domain.PointInfo;
import com.tcctyn.iot.forestfire.enums.DeviceStatusEnum;
import com.tcctyn.iot.forestfire.enums.DeviceTypeEnum;
import com.tcctyn.iot.forestfire.enums.PointTypeEnum;
import com.tcctyn.iot.forestfire.mapper.DeviceInfoMapper;
import com.tcctyn.iot.forestfire.mapper.PointInfoMapper;
import com.tcctyn.iot.forestfire.searcher.BaseRegionSearcher;
import com.tcctyn.iot.forestfire.searcher.deviceinfo.DeviceOwnershipSearcher;
import com.tcctyn.iot.forestfire.service.ICommonService;
import com.tcctyn.iot.forestfire.service.IDeviceChanelsService;
import com.tcctyn.iot.forestfire.service.IDeviceInfoService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static com.tcctyn.common.core.utils.DateUtils.getNowDate;
import static com.tcctyn.common.core.utils.SecurityUtils.getUsername;

/**
 * <p>
 * 设备信息表 服务实现类
 * </p>
 *
 * @author 代超
 * @since 2024-10-09
 */
@Service
public class DeviceInfoServiceImpl extends ServiceImpl<DeviceInfoMapper, DeviceInfo> implements IDeviceInfoService {


    @Resource
    private DeviceInfoMapper deviceInfoMapper;

    @Resource
    private PointInfoMapper pointInfoMapper;

    @Autowired
    private ICommonService commonService;

    @Autowired
    private IDeviceChanelsService deviceChanelsService;

    @Override
    public Page<DeviceInfo> list(Integer pageNo, Integer pageSize) {
        return deviceInfoMapper.selectPage(new Page<>(pageNo, pageSize), new QueryWrapper<>());
    }

    @Override
    public DeviceInfoVO getById(Long id) {
        DeviceInfo deviceInfo = deviceInfoMapper.selectById(id);
        //处理regionName、pointName、airportName
        DeviceInfoVO deviceInfoVO = new DeviceInfoVO();
        BeanUtils.copyProperties(deviceInfo, deviceInfoVO);
        Map<Long, String> regionIdToNameMap = commonService.regionMapIdToName(deviceInfoVO.getRegionId());
        deviceInfoVO.setRegionName(regionIdToNameMap.get(deviceInfoVO.getRegionId()));
        if (Objects.nonNull(deviceInfoVO.getPointId())) {
            deviceInfoVO.setPointName(commonService.pointMapIdToName(deviceInfoVO.getPointId()).get(deviceInfoVO.getPointId()));
        }
        //如果是无人机，需要返回机场名称
        if (Objects.nonNull(deviceInfoVO.getAirportId())) {
            DeviceInfo airportInfo = deviceInfoMapper.selectOne(new QueryWrapper<DeviceInfo>().lambda().eq(DeviceInfo::getId, deviceInfoVO.getAirportId()));
            deviceInfoVO.setAirportName(airportInfo.getDeviceName());
        }
        return deviceInfoVO;
    }

    @Override
    public Integer create(DeviceInfo deviceInfo) {
        //校验设备编号的唯一性
        List<DeviceInfo> deviceInfoList = deviceInfoMapper.selectList(new QueryWrapper<DeviceInfo>().eq("device_no", deviceInfo.getDeviceNo()));
        if (!CollectionUtils.isEmpty(deviceInfoList)) {
            throw new ServiceException("设备编号已存在");
        }
        //如果是云台或者卡口，自动添加通道信息
        if (DeviceTypeEnum.CLOUDTERRACE.name().equals(deviceInfo.getDeviceType()) || DeviceTypeEnum.BAYONET.name().equals(deviceInfo.getDeviceType())) {
            // 自动创建关联通道
            int[] suffixes = {101, 102, 201, 202}; // 定义需要循环的通道后缀
            for (int suffix : suffixes) {
                DeviceChanels deviceChannel = new DeviceChanels();
                deviceChannel.setDeviceNo(deviceInfo.getDeviceNo());
                deviceChannel.setName("自动增加通道");
                // 使用循环变量动态生成streamUrl
                // 后续需更改
                deviceChannel.setStreamUrl("http://47.109.202.121:11080/live" + deviceInfo.getDeviceNo() + "/" + suffix + ".flv");
                deviceChannel.setCreatedBy(getUsername());
                deviceChannel.setCreatedTime(getNowDate());
                deviceChanelsService.create(deviceChannel);
            }
        }

        // 如果是机场，需要选择机场位置，带上经纬度信息
        if (DeviceTypeEnum.AIRPORT.name().equals(deviceInfo.getDeviceType())) {
            if (Objects.isNull(deviceInfo.getRegionId())) {
                throw new ServiceException("请选择机场所在林区/林场");
            }
            if (Objects.isNull(deviceInfo.getAddress())) {
                throw new ServiceException("请选择机场地址");
            }
            if (Objects.isNull(deviceInfo.getLongitude())) {
                throw new ServiceException("请输入机场经度");
            }
            if (Objects.isNull(deviceInfo.getLatitude())) {
                throw new ServiceException("请输入机场纬度");
            }
        }
        if (DeviceTypeEnum.DRONE.name().equals(deviceInfo.getDeviceType())) {
            if (Objects.isNull(deviceInfo.getAirportId())) {
                throw new ServiceException("请选择机场");
            }
            //绑定机场信息，无人机默认挂在机场所在的区域
            DeviceInfo airportInfo = deviceInfoMapper.selectById(deviceInfo.getAirportId());
            if (Objects.nonNull(airportInfo)) {
                deviceInfo.setRegionId(airportInfo.getRegionId());
                deviceInfo.setForestId(airportInfo.getForestId());
                deviceInfo.setFarmId(airportInfo.getFarmId());
                deviceInfo.setFenceId(airportInfo.getFenceId());
            }
        }
        // 添加设备的时候需要将 林区、林场、电子围栏的id 冗余到数据库表中，方便后续的查询
        PointInfo pointInfo = pointInfoMapper.selectById(deviceInfo.getPointId());
        if (Objects.nonNull(pointInfo)) {
            deviceInfo.setForestId(pointInfo.getForestId());
            deviceInfo.setFarmId(pointInfo.getFarmId());
            deviceInfo.setFenceId(pointInfo.getFenceId());
        }
        return deviceInfoMapper.insert(deviceInfo);
    }

    @Override
    public Integer delete(Long id) {
        //TODO 如果删除的设备是机场，那么需要删除机场下面的所有无人机
        return deviceInfoMapper.deleteById(id);
    }

    @Override
    public Integer update(DeviceInfo deviceInfo) {
        //校验设备编号的唯一性
        List<DeviceInfo> deviceInfoList = deviceInfoMapper.selectList(new QueryWrapper<DeviceInfo>()
                .eq("device_no", deviceInfo.getDeviceNo())
                .ne("id", deviceInfo.getId())
        );
        if (!CollectionUtils.isEmpty(deviceInfoList)) {
            throw new ServiceException("设备编号已存在");
        }
        // 如果是机场，需要选择机场位置，带上经纬度信息
        if (DeviceTypeEnum.AIRPORT.name().equals(deviceInfo.getDeviceType())) {
            if (Objects.isNull(deviceInfo.getRegionId())) {
                throw new ServiceException("请选择机场所在林区/林场");
            }
            if (Objects.isNull(deviceInfo.getAddress())) {
                throw new ServiceException("请选择机场地址");
            }
            if (Objects.isNull(deviceInfo.getLongitude())) {
                throw new ServiceException("请输入机场经度");
            }
            if (Objects.isNull(deviceInfo.getLatitude())) {
                throw new ServiceException("请输入机场纬度");
            }
        }
        if (DeviceTypeEnum.DRONE.name().equals(deviceInfo.getDeviceType())) {
            if (Objects.isNull(deviceInfo.getAirportId())) {
                throw new ServiceException("请选择机场");
            }
            //绑定机场信息，无人机默认挂在机场所在的区域
            DeviceInfo airportInfo = deviceInfoMapper.selectById(deviceInfo.getAirportId());
            if (Objects.nonNull(airportInfo)) {
                deviceInfo.setRegionId(airportInfo.getRegionId());
                deviceInfo.setForestId(airportInfo.getForestId());
                deviceInfo.setFarmId(airportInfo.getFarmId());
                deviceInfo.setFenceId(airportInfo.getFenceId());
            }
        }

        // 添加设备的时候需要将 林区、林场、电子围栏的id 冗余到数据库表中，方便后续的查询
        PointInfo pointInfo = pointInfoMapper.selectById(deviceInfo.getPointId());
        if (Objects.nonNull(pointInfo)) {
            deviceInfo.setForestId(pointInfo.getForestId());
            deviceInfo.setFarmId(pointInfo.getFarmId());
            deviceInfo.setFenceId(pointInfo.getFenceId());
        }
        return deviceInfoMapper.updateById(deviceInfo);
    }

    @Override
    public List<DeviceInfo> selectList(DeviceInfo deviceInfo) {
        // 设备分页查询这里可以根据林区、林场、电子围栏id进行过滤
        List<DeviceInfo> deviceInfoList = deviceInfoMapper.selectList(new QueryWrapper<DeviceInfo>()
                .lambda()
                .like(StringUtils.isNotBlank(deviceInfo.getDeviceNo()), DeviceInfo::getDeviceNo, deviceInfo.getDeviceNo())
                .eq(StringUtils.isNotBlank(deviceInfo.getDeviceType()), DeviceInfo::getDeviceType, deviceInfo.getDeviceType())
                .eq(StringUtils.isNotBlank(deviceInfo.getDeviceManufacturer()), DeviceInfo::getDeviceManufacturer, deviceInfo.getDeviceManufacturer())
                .eq(Objects.nonNull(deviceInfo.getPointId()), DeviceInfo::getPointId, deviceInfo.getPointId())
                .eq(Objects.nonNull(deviceInfo.getForestId()), DeviceInfo::getForestId, deviceInfo.getForestId())
                .eq(Objects.nonNull(deviceInfo.getFarmId()), DeviceInfo::getFarmId, deviceInfo.getFarmId())
                .eq(Objects.nonNull(deviceInfo.getFenceId()), DeviceInfo::getFenceId, deviceInfo.getFenceId())
                .eq(Objects.nonNull(deviceInfo.getEnableFlag()), DeviceInfo::getEnableFlag, deviceInfo.getEnableFlag())
                .eq(Objects.nonNull(deviceInfo.getDeviceStatus()), DeviceInfo::getDeviceStatus, deviceInfo.getDeviceStatus())
                .orderByDesc(DeviceInfo::getCreatedTime)
        );
        return deviceInfoList;
    }

    @Override
    public DeviceAndPointInfoVO selectByDeviceNo(String deviceNo) {
        DeviceAndPointInfoVO vo = deviceInfoMapper.selectByDeviceNo(deviceNo);
        return vo;
    }

    @Override
    public DeviceStatisticsVO deviceStatistics(DeviceInfo deviceInfo) {
        // 1、查询设备数量；2、查询云台、卡口数量
        List<DeviceInfo> deviceInfoList = deviceInfoMapper.selectList(new QueryWrapper<DeviceInfo>()
                .lambda()
                .eq(Objects.nonNull(deviceInfo.getForestId()), DeviceInfo::getForestId, deviceInfo.getForestId())
                .eq(Objects.nonNull(deviceInfo.getFarmId()), DeviceInfo::getFarmId, deviceInfo.getFarmId())
                .eq(Objects.nonNull(deviceInfo.getFenceId()), DeviceInfo::getFenceId, deviceInfo.getFenceId())
        );
        //查询云台与卡口数量
        List<PointInfo> pointInfoList = pointInfoMapper.selectList(new QueryWrapper<PointInfo>()
                .lambda()
                .eq(Objects.nonNull(deviceInfo.getForestId()), PointInfo::getForestId, deviceInfo.getForestId())
                .eq(Objects.nonNull(deviceInfo.getFarmId()), PointInfo::getFarmId, deviceInfo.getFarmId())
                .eq(Objects.nonNull(deviceInfo.getFenceId()), PointInfo::getFenceId, deviceInfo.getFenceId())
                .and(wrapper -> wrapper.eq(PointInfo::getPointType, PointTypeEnum.PTZ.name())
                        .or().eq(PointInfo::getPointType, PointTypeEnum.BAYONET.name())
                ));
        Map<String, List<PointInfo>> pointMap = pointInfoList.stream().collect(Collectors.groupingBy(PointInfo::getPointType));
        List<Long> ptzIdList = new ArrayList<>();
        List<Long> bayonetIdList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(pointMap)) {
            //获取云台数量
            List<PointInfo> ptzPointList = pointMap.get(PointTypeEnum.PTZ.name()) == null ? new ArrayList<>() : pointMap.get(PointTypeEnum.PTZ.name());
            //获取卡口数量
            List<PointInfo> bayonetPointList = pointMap.get(PointTypeEnum.BAYONET.name()) == null ? new ArrayList<>() : pointMap.get(PointTypeEnum.BAYONET.name());

            ptzIdList = ptzPointList.stream().map(PointInfo::getId).collect(Collectors.toList());
            bayonetIdList = bayonetPointList.stream().map(PointInfo::getId).collect(Collectors.toList());
        }
        //云台设备数量
        Integer ptzOnlineNum = 0;
        Integer ptzOfflineNum = 0;
        Integer ptzTotalNum = 0;

        //卡口设备数量
        Integer bayonetOnlineNum = 0;
        Integer bayonetOfflineNum = 0;
        Integer bayonetTotalNum = 0;

        //无人机数量
        Integer droneTotalNum = 0;
        Integer droneOnlineNum = 0;
        Integer droneOfflineNum = 0;

        //其他设备数量
        Integer otherOnlineNum = 0;
        Integer otherOfflineNum = 0;
        Integer otherTotalNum = 0;

        for (DeviceInfo device : deviceInfoList) {
            if (ptzIdList.contains(device.getPointId())) {
                //云台设备
                if (DeviceStatusEnum.ON.name().equals(device.getDeviceStatus())) {
                    ptzOnlineNum++;
                } else {
                    ptzOfflineNum++;
                }
            } else if (bayonetIdList.contains(device.getPointId())) {
                //卡口设备
                if (DeviceStatusEnum.ON.name().equals(device.getDeviceStatus())) {
                    bayonetOnlineNum++;
                } else {
                    bayonetOfflineNum++;
                }
            } else if (DeviceTypeEnum.DRONE.name().equals(device.getDeviceType())) {
                // 无人机设备
                droneTotalNum++;
                if (DeviceStatusEnum.ON.name().equals(device.getDeviceStatus())) {
                    droneOnlineNum++;
                } else {
                    droneOfflineNum++;
                }

            } else {
                //其他设备
                if (DeviceStatusEnum.ON.name().equals(device.getDeviceStatus())) {
                    otherOnlineNum++;
                } else {
                    otherOfflineNum++;
                }
            }
        }
        //无人机在线率
        String droneOnlineRate = droneTotalNum == 0 ? "0%" : String.format("%.2f%%", droneOnlineNum * 1.0 / droneTotalNum * 100);

        ptzTotalNum = ptzOnlineNum + ptzOfflineNum;
        bayonetTotalNum = bayonetOnlineNum + bayonetOfflineNum;
        otherTotalNum = otherOnlineNum + otherOfflineNum;

        DeviceStatisticsVO vo = new DeviceStatisticsVO();
        List<DeviceStatisticsSubVO> subList = new ArrayList<>();
        //无人机数据
        DeviceStatisticsSubVO droneVO = new DeviceStatisticsSubVO();
        droneVO.setType("drone");
        droneVO.setOnlineNum(droneOnlineNum);
        droneVO.setOfflineNum(droneOfflineNum);
        droneVO.setTotalNum(droneTotalNum);
        droneVO.setOnlineRate(droneOnlineRate);
        subList.add(droneVO);
        //云台数据
        DeviceStatisticsSubVO ptzVO = new DeviceStatisticsSubVO();
        ptzVO.setType("ptz");
        ptzVO.setOnlineNum(ptzOnlineNum);
        ptzVO.setOfflineNum(ptzOfflineNum);
        ptzVO.setTotalNum(ptzTotalNum);
        ptzVO.setOnlineRate(String.format("%.2f%%", ptzTotalNum == 0 ? 0 : ptzVO.getOnlineNum() * 1.0 / ptzTotalNum * 100));
        subList.add(ptzVO);
        //卡口数据
        DeviceStatisticsSubVO bayonetVO = new DeviceStatisticsSubVO();
        bayonetVO.setType("bayonet");
        bayonetVO.setOnlineNum(bayonetOnlineNum);
        bayonetVO.setOfflineNum(bayonetOfflineNum);
        bayonetVO.setTotalNum(bayonetTotalNum);
        bayonetVO.setOnlineRate(String.format("%.2f%%", bayonetTotalNum == 0 ? 0 : bayonetVO.getOnlineNum() * 1.0 / bayonetTotalNum * 100));
        subList.add(bayonetVO);
        //其他设备数据
        DeviceStatisticsSubVO otherVO = new DeviceStatisticsSubVO();
        otherVO.setType("other");
        otherVO.setOnlineNum(otherOnlineNum);
        otherVO.setOfflineNum(otherOfflineNum);
        otherVO.setTotalNum(otherTotalNum);
        otherVO.setOnlineRate(String.format("%.2f%%", otherTotalNum == 0 ? 0 : otherVO.getOnlineNum() * 1.0 / otherTotalNum * 100));
        subList.add(otherVO);

        vo.setSubList(subList);
        vo.setTotal(droneTotalNum + ptzTotalNum + bayonetTotalNum + otherTotalNum);
        vo.setOverallOnlineRate(String.format("%.2f%%", vo.getTotal() == 0 ? 0 : (droneOnlineNum + ptzOnlineNum + bayonetOnlineNum + otherOnlineNum) * 1.0 / vo.getTotal() * 100));
        return vo;
    }

    @Override
    public Integer updateEnableFlag(DeviceInfo deviceInfo) {
        Integer result = deviceInfoMapper.updateById(deviceInfo);
        return result;
    }

    @Override
    public Integer deleteBatch(List<Long> idList) {
        //TODO 如果删除的设备是机场，那么需要删除机场下面的所有无人机
        return deviceInfoMapper.deleteBatchIds(idList);
    }

    @Override
    public List<DeviceOwnerVO> queryOwnership(DeviceOwnershipSearcher searcher) {
        String deviceType = searcher.getDeviceType();
        //无人机对应机场
        if (DeviceTypeEnum.DRONE.name().equals(deviceType)) {
            List<DeviceInfo> airportList = deviceInfoMapper.selectList(new QueryWrapper<DeviceInfo>().lambda()
                    .eq(DeviceInfo::getDeviceType, DeviceTypeEnum.AIRPORT.name())
                    .eq(Objects.nonNull(searcher.getForestId()), DeviceInfo::getForestId, searcher.getForestId())
                    .eq(Objects.nonNull(searcher.getFarmId()), DeviceInfo::getFarmId, searcher.getFarmId())
                    .eq(Objects.nonNull(searcher.getFenceId()), DeviceInfo::getFenceId, searcher.getFenceId())
            );
            return airportList.stream().map(airport -> new DeviceOwnerVO(airport.getId(), airport.getDeviceName())).collect(Collectors.toList());
        }

        //水泵对应水源地
        if (DeviceTypeEnum.PUMP.name().equals(deviceType)) {
            List<PointInfo> waterSourceList = pointInfoMapper.selectList(new QueryWrapper<PointInfo>().lambda()
                    .eq(PointInfo::getPointType, PointTypeEnum.WATER_SOURCE.name())
                    .eq(Objects.nonNull(searcher.getForestId()), PointInfo::getForestId, searcher.getForestId())
                    .eq(Objects.nonNull(searcher.getFarmId()), PointInfo::getFarmId, searcher.getFarmId())
                    .eq(Objects.nonNull(searcher.getFenceId()), PointInfo::getFenceId, searcher.getFenceId())
            );
            return waterSourceList.stream().map(waterSource -> new DeviceOwnerVO(waterSource.getId(), waterSource.getPointName())).collect(Collectors.toList());
        }

        //摄像头、红外设备、气象监测仪对应云台、卡口
        if (DeviceTypeEnum.CAMERA.name().equals(deviceType) || DeviceTypeEnum.INFRARED.name().equals(deviceType) || DeviceTypeEnum.METEOROLOGIC.name().equals(deviceType)) {
            List<PointInfo> ptzAndBayonetList = pointInfoMapper.selectList(new QueryWrapper<PointInfo>().lambda()
                    .in(PointInfo::getPointType, Arrays.asList(PointTypeEnum.BAYONET.name(), PointTypeEnum.PTZ.name()))
                    .eq(Objects.nonNull(searcher.getForestId()), PointInfo::getForestId, searcher.getForestId())
                    .eq(Objects.nonNull(searcher.getFarmId()), PointInfo::getFarmId, searcher.getFarmId())
                    .eq(Objects.nonNull(searcher.getFenceId()), PointInfo::getFenceId, searcher.getFenceId())
                    .orderByAsc(PointInfo::getPointType)
            );
            return ptzAndBayonetList.stream().map(ptzAndBayonet -> new DeviceOwnerVO(ptzAndBayonet.getId(), ptzAndBayonet.getPointName())).collect(Collectors.toList());
        }
        return Collections.emptyList();
    }

    @Override
    public DeviceStatisticsByStatusVO statisticsByStatus() {
        DeviceStatisticsByStatusVO vo = deviceInfoMapper.statisticsByStatus();
        return vo;
    }

    @Override
    public DeviceStatisticsByPointVO deviceStatisticsByPoint(BaseRegionSearcher searcher) {
        //实现思路，先查询出林区、林场、电子围栏下的云台、卡口数量；分成云台、卡口；再分别查询云台、卡口下的设备数量，
        List<PointInfo> pointInfoList = pointInfoMapper.selectList(new QueryWrapper<PointInfo>().lambda()
                .in(PointInfo::getPointType, Arrays.asList(PointTypeEnum.BAYONET.name(), PointTypeEnum.PTZ.name()))
                .eq(Objects.nonNull(searcher.getForestId()), PointInfo::getForestId, searcher.getForestId())
                .eq(Objects.nonNull(searcher.getFarmId()), PointInfo::getFarmId, searcher.getFarmId())
                .eq(Objects.nonNull(searcher.getFenceId()), PointInfo::getFenceId, searcher.getFenceId())
        );
        Integer totalNum = 0;
        Integer onlineNum = 0;
        Integer ptzNum = 0;
        Integer bayonetNum = 0;
        String onlineRate = "0.00%";
        if (!CollectionUtils.isEmpty(pointInfoList)) {
            List<Long> pointIdList = pointInfoList.stream().map(PointInfo::getId).collect(Collectors.toList());
            Map<Long, String> pointIdAndTypeMap = pointInfoList.stream().collect(Collectors.toMap(PointInfo::getId, PointInfo::getPointType));
            //查询云台、卡口下的设备数量
            List<DeviceInfo> deviceInfoList = deviceInfoMapper.selectList(new QueryWrapper<DeviceInfo>().lambda()
                    .in(DeviceInfo::getPointId, pointIdList)
                    .eq(DeviceInfo::getEnableFlag, 1)
            );
            if (!CollectionUtils.isEmpty(deviceInfoList)) {
                for (DeviceInfo deviceInfo : deviceInfoList) {
                    if (DeviceStatusEnum.ON.name().equals(deviceInfo.getDeviceStatus())) {
                        onlineNum++;
                    }
                    if (PointTypeEnum.BAYONET.name().equals(pointIdAndTypeMap.get(deviceInfo.getPointId()))) {
                        bayonetNum++;
                    }
                    if (PointTypeEnum.PTZ.name().equals(pointIdAndTypeMap.get(deviceInfo.getPointId()))) {
                        ptzNum++;
                    }
                }
            }
        }
        totalNum = ptzNum + bayonetNum;
        onlineRate = String.format("%.2f%s", onlineNum == 0 ? 0 : onlineNum * 1.0 / totalNum * 100, "%");

        DeviceStatisticsByPointVO deviceStatisticsByPointVO = new DeviceStatisticsByPointVO();
        deviceStatisticsByPointVO.setTotalNum(totalNum);
        deviceStatisticsByPointVO.setOnlineNum(onlineNum);
        deviceStatisticsByPointVO.setOnlineRate(onlineRate);
        deviceStatisticsByPointVO.setBayonetNum(bayonetNum);
        deviceStatisticsByPointVO.setPtzNum(ptzNum);
        return deviceStatisticsByPointVO;
    }

    @Override
    public List<AirportVO> selectAirport(BaseRegionSearcher searcher) {
        List<DeviceInfo> deviceInfoList = deviceInfoMapper.selectList(new QueryWrapper<DeviceInfo>().lambda()
                .eq(Objects.nonNull(searcher.getForestId()), DeviceInfo::getForestId, searcher.getForestId())
                .eq(Objects.nonNull(searcher.getFarmId()), DeviceInfo::getFarmId, searcher.getFarmId())
                .eq(Objects.nonNull(searcher.getFenceId()), DeviceInfo::getFenceId, searcher.getFenceId())
                .eq(DeviceInfo::getDeviceType, DeviceTypeEnum.AIRPORT.name())
        );
        List<AirportVO> airportVOList = BeanUtils.copyList(deviceInfoList, AirportVO.class);
        Map<Long, String> regionIdAndNameMap = commonService.regionMapIdToName(null);
        Map<Long, String> airportIdAndNameMap = airportVOList.stream().collect(Collectors.toMap(AirportVO::getId, AirportVO::getDeviceName));
        if (!CollectionUtils.isEmpty(airportVOList)) {
            List<Long> airportIdList = airportVOList.stream().map(AirportVO::getId).collect(Collectors.toList());
            // 将机场下的无人机列表挂出来
            List<DeviceInfo> droneList = deviceInfoMapper.selectList(new QueryWrapper<DeviceInfo>().lambda()
                    .eq(DeviceInfo::getDeviceType, DeviceTypeEnum.DRONE.name())
                    .in(DeviceInfo::getAirportId, airportIdList)
            );
            List<DroneVO> droneVOList = BeanUtils.copyList(droneList, DroneVO.class);
            droneVOList.forEach(droneVO -> {
                droneVO.setAirportName(airportIdAndNameMap.get(droneVO.getAirportId()) == null ? "" : airportIdAndNameMap.get(droneVO.getAirportId()));
                droneVO.setRegionName(regionIdAndNameMap.get(droneVO.getRegionId()) == null ? "" : regionIdAndNameMap.get(droneVO.getRegionId()));
            });
            Map<Long, List<DroneVO>> airportAndDroneMap = droneVOList.stream().collect(Collectors.groupingBy(DroneVO::getAirportId));

            airportVOList.forEach(airportVO -> {
                airportVO.setRegionName(regionIdAndNameMap.get(airportVO.getRegionId()) == null ? "" : regionIdAndNameMap.get(airportVO.getRegionId()));
                airportVO.setDroneList(airportAndDroneMap.get(airportVO.getId()) == null ? new ArrayList<>() : airportAndDroneMap.get(airportVO.getId()));
            });
        }


        return airportVOList;
    }

    @Override
    public List<DeviceInfo> selectDroneByAirportId(Long airportId) {
        List<DeviceInfo> deviceInfoList = deviceInfoMapper.selectList(new QueryWrapper<DeviceInfo>().lambda()
                .eq(DeviceInfo::getAirportId, airportId)
                .eq(DeviceInfo::getDeviceType, DeviceTypeEnum.DRONE.name())
        );
        return deviceInfoList;
    }

    @Override
    public List<DeviceInfo> selectListByPointList(List<Long> pointIdList) {
        List<DeviceInfo> deviceInfoList = deviceInfoMapper.selectList(new QueryWrapper<DeviceInfo>().lambda()
                .in(DeviceInfo::getPointId, pointIdList)
        );
        return deviceInfoList;
    }

    @Override
    public List<CameraVO> selectCamera(BaseRegionSearcher searcher) {
        List<DeviceInfo> deviceInfoList = deviceInfoMapper.selectList(new QueryWrapper<DeviceInfo>().lambda()
                .eq(Objects.nonNull(searcher.getForestId()), DeviceInfo::getForestId, searcher.getForestId())
                .eq(Objects.nonNull(searcher.getFarmId()), DeviceInfo::getFarmId, searcher.getFarmId())
                .eq(Objects.nonNull(searcher.getFenceId()), DeviceInfo::getFenceId, searcher.getFenceId())
                .eq(DeviceInfo::getDeviceType, DeviceTypeEnum.CAMERA.name())
                .eq(DeviceInfo::getEnableFlag, 1)
                .orderByDesc(DeviceInfo::getCreatedTime)
        );
        List<CameraVO> cameraVOList = BeanUtils.copyList(deviceInfoList, CameraVO.class);
        Map<Long, String> regionIdAndNameMap = commonService.regionMapIdToName(null);
        cameraVOList.forEach(cameraVO -> cameraVO.setRegionName(Optional.ofNullable(regionIdAndNameMap.get(cameraVO.getRegionId())).orElse("")));
        return cameraVOList;
    }

}
