package com.tcctyn.iot.forestfire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctyn.common.core.constant.RegionConstants;
import com.tcctyn.common.core.web.domain.entity.RegionInfo;
import com.tcctyn.common.core.exception.ServiceException;
import com.tcctyn.common.core.utils.StringUtils;
import com.tcctyn.iot.forestfire.VO.FireReportStatisticsVO;
import com.tcctyn.iot.forestfire.domain.FireReport;
import com.tcctyn.iot.forestfire.domain.FireWarningInfo;
import com.tcctyn.iot.forestfire.enums.FireWarningDisposalStatusEnum;
import com.tcctyn.iot.forestfire.enums.ReportMethodEnum;
import com.tcctyn.iot.forestfire.mapper.FireReportMapper;
import com.tcctyn.iot.forestfire.mapper.FireWarningInfoMapper;
import com.tcctyn.iot.forestfire.mapper.RegionInfoMapper;
import com.tcctyn.iot.forestfire.service.IFireReportService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 火情上报表 服务实现类
 * </p>
 *
 * @author 代超
 * @since 2024-11-01
 */
@Service
public class FireReportServiceImpl extends ServiceImpl<FireReportMapper, FireReport> implements IFireReportService {


    @Resource
    private FireReportMapper fireReportMapper;

    @Resource
    private RegionInfoMapper regionInfoMapper;

    @Resource
    private FireWarningInfoMapper fireWarningInfoMapper;

    @Override
    public Page<FireReport> list(Integer pageNo, Integer pageSize) {
        return fireReportMapper.selectPage(new Page<>(pageNo, pageSize), new QueryWrapper<>());
    }

    @Override
    public FireReport getById(Long id) {
        FireReport fireReport = fireReportMapper.selectById(id);
        return fireReport;
    }

    @Override
    public Integer create(FireReport fireReport) {
        //后台上报的，默认设置为人工上报
        fireReport.setReportMethod(ReportMethodEnum.MANUAL_REPORTING.name());
        Long regionId = fireReport.getRegionId();
        if (Objects.nonNull(regionId)) {
            RegionInfo regionInfo = regionInfoMapper.selectById(regionId);
            String ancestors = regionInfo.getAncestors();
            String[] ancestorsArr = ancestors.split(RegionConstants.DIVISION_COMMA);
            switch (ancestorsArr.length) {
                case 1:
                    fireReport.setForestId(regionId);
                    break;
                case 2:
                    fireReport.setForestId(Long.valueOf(ancestorsArr[1]));
                    fireReport.setFarmId(regionId);
                    break;
                case 3:
                    fireReport.setForestId(Long.valueOf(ancestorsArr[1]));
                    fireReport.setFarmId(Long.valueOf(ancestorsArr[2]));
                    fireReport.setFenceId(regionId);
                    break;
                case 4:
                    fireReport.setForestId(Long.valueOf(ancestorsArr[1]));
                    fireReport.setFarmId(Long.valueOf(ancestorsArr[2]));
                    fireReport.setFenceId(Long.valueOf(ancestorsArr[3]));
                    break;
            }
        }
        //如果火情预警id不为空，需要将该火情预警的处置状态调整为已上报
        if (Objects.nonNull(fireReport.getFireWarningId())) {
            FireWarningInfo fireWarningInfo = new FireWarningInfo();
            fireWarningInfo.setId(fireReport.getFireWarningId());
            fireWarningInfo.setDisposalStatus(FireWarningDisposalStatusEnum.REPORTED.name());
            fireWarningInfo.setDisposalTime(new Date());
            fireWarningInfo.setDisposalBy(fireReport.getReporterId());
            fireWarningInfoMapper.updateById(fireWarningInfo);
        }
        return fireReportMapper.insert(fireReport);
    }

    @Override
    public Integer delete(Long id) {
        return fireReportMapper.deleteById(id);
    }

    @Override
    public Integer update(FireReport fireReport) {
        return fireReportMapper.updateById(fireReport);
    }

    @Override
    public List<FireReport> selectList(FireReport fireReport) {
        return fireReportMapper.selectList(new QueryWrapper<FireReport>().lambda()
                .like(StringUtils.isNotEmpty(fireReport.getFireTitle()), FireReport::getFireTitle, fireReport.getFireTitle())
                .eq(Objects.nonNull(fireReport.getRegionId()), FireReport::getRegionId, fireReport.getRegionId())
                .eq(StringUtils.isNotEmpty(fireReport.getFireHazardLevel()), FireReport::getFireHazardLevel, fireReport.getFireHazardLevel())
                .eq(Objects.nonNull(fireReport.getDisposalStatus()), FireReport::getDisposalStatus, fireReport.getDisposalStatus())
                .eq(StringUtils.isNotEmpty(fireReport.getReportMethod()), FireReport::getReportMethod, fireReport.getReportMethod())
                .eq(Objects.nonNull(fireReport.getArchiveStatus()), FireReport::getArchiveStatus, fireReport.getArchiveStatus())
                .orderByDesc(FireReport::getCreatedTime)
        );
    }

    @Override
    public Integer archive(FireReport fireReport) {
        if (StringUtils.isEmpty(fireReport.getFirefightingSituation())) {
            throw new ServiceException("请填写扑救情况");
        }
        if (StringUtils.isEmpty(fireReport.getDisasterSituation())) {
            throw new ServiceException("请填写灾后情况说明");
        }
        if (StringUtils.isEmpty(fireReport.getFirefightingUnit())) {
            throw new ServiceException("请填写扑救单位");
        }
        if (StringUtils.isEmpty(fireReport.getPersonInCharge())) {
            throw new ServiceException("请填写主要负责人");
        }
        //火情上报归档
        fireReport.setArchiveStatus(1);
        fireReport.setArchiveTime(new Date());
        return fireReportMapper.updateById(fireReport);
    }

    @Override
    public FireReportStatisticsVO fireReportStatistics() {
        FireReportStatisticsVO vo = fireReportMapper.fireReportStatistics();
        return vo;
    }

    @Override
    public Integer disposal(FireReport fireReport) {
        if (StringUtils.isEmpty(fireReport.getDisposalInstruction())) {
            throw new ServiceException("请填写处置说明");
        }
        fireReport.setDisposalStatus(1);
        return fireReportMapper.updateById(fireReport);
    }

}
