package com.tcctyn.iot.forestfire.service.impl;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctyn.common.core.exception.ServiceException;
import com.tcctyn.iot.forestfire.domain.ApprovalRecord;
import com.tcctyn.iot.forestfire.domain.LeaveRecord;
import com.tcctyn.iot.forestfire.enums.ApprovalBusinessTypeEnum;
import com.tcctyn.iot.forestfire.mapper.ApprovalRecordMapper;
import com.tcctyn.iot.forestfire.mapper.LeaveRecordMapper;
import com.tcctyn.iot.forestfire.service.ICommonService;
import com.tcctyn.iot.forestfire.service.ILeaveRecordService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 请假记录表 服务实现类
 * </p>
 *
 * @author daichao
 * @since 2024-12-09
 */
@Service
public class LeaveRecordServiceImpl extends ServiceImpl<LeaveRecordMapper, LeaveRecord> implements ILeaveRecordService {


    @Resource
    private LeaveRecordMapper leaveRecordMapper;

    @Resource
    private ApprovalRecordMapper approvalRecordMapper;

    @Autowired
    private ICommonService commonService;

    @Override
    public Page<LeaveRecord> list(Integer pageNo, Integer pageSize) {
        return leaveRecordMapper.selectPage(new Page<>(pageNo, pageSize), new QueryWrapper<>());
    }

    @Override
    public LeaveRecord getById(Long id) {
        LeaveRecord leaveRecord = leaveRecordMapper.selectById(id);
        return leaveRecord;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Integer create(LeaveRecord leaveRecord) {
        Integer result = leaveRecordMapper.insert(leaveRecord);
        Long id = leaveRecord.getId();
        //此处需要获取到插入的业务数据id
        //新增请假数据后，需要同步写入到审批记录表
        ApprovalRecord approvalRecord = new ApprovalRecord();
        approvalRecord.setBusinessType(ApprovalBusinessTypeEnum.LEAVE.name());
        approvalRecord.setBusinessDataId(id);
        //业务数据内容需要保存userName 跟 deptName, 方便前端进行展示 TODO 这里后续可以优化下，从用户登录信息中取name
        Map<Long, String> userIdToNameMap = commonService.userMapIdToName(leaveRecord.getUserId());
        Map<Long, String> deptIdToNameMap = commonService.deptMapIdToName(leaveRecord.getDeptId());
        JSONObject businessContent = new JSONObject();
        businessContent.put("userId", leaveRecord.getUserId());
        businessContent.put("deptId", leaveRecord.getDeptId());
        businessContent.put("userName", userIdToNameMap.get(leaveRecord.getUserId()));
        businessContent.put("deptName", deptIdToNameMap.get(leaveRecord.getDeptId()));
        businessContent.put("leaveType", leaveRecord.getLeaveType());
        businessContent.put("startTime", leaveRecord.getStartTime());
        businessContent.put("endTime", leaveRecord.getEndTime());
        businessContent.put("leaveDuration", leaveRecord.getLeaveDuration());
        businessContent.put("leaveInstruction", leaveRecord.getLeaveInstruction());
        businessContent.put("picture", leaveRecord.getPicture());
        approvalRecord.setBusinessContent(businessContent);
        approvalRecordMapper.insert(approvalRecord);
        return result;
    }

    @Override
    public Integer delete(Long id) {
        return leaveRecordMapper.deleteById(id);
    }

    @Override
    public Integer update(LeaveRecord leaveRecord) {
        return leaveRecordMapper.updateById(leaveRecord);
    }

    @Override
    public List<LeaveRecord> selectList(LeaveRecord leaveRecord) {
        List<LeaveRecord> leaveRecordList = leaveRecordMapper.selectList(new QueryWrapper<>(leaveRecord).lambda()
                .eq(StringUtils.isNotBlank(leaveRecord.getLeaveType()), LeaveRecord::getLeaveType, leaveRecord.getLeaveType())
                .eq(LeaveRecord::getUserId, leaveRecord.getUserId())
                .orderByAsc(LeaveRecord::getApprovalFlag)
                .orderByDesc(LeaveRecord::getCreatedTime)
        );
        return leaveRecordList;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Integer revoke(Long id) {
        LeaveRecord leaveRecord = leaveRecordMapper.selectById(id);
        if (1 == leaveRecord.getApprovalFlag()) {
            throw new ServiceException("已审批，不能撤销");
        }
        leaveRecord.setRevokeFlag(1);
        leaveRecord.setRevokeTime(new Date());
        Integer result = leaveRecordMapper.updateById(leaveRecord);
        //撤销后，需要同步更新到审批记录，删除该条业务数据的审批记录
        approvalRecordMapper.delete(new QueryWrapper<ApprovalRecord>().lambda()
                .eq(ApprovalRecord::getBusinessDataId, id)
        );
        return result;
    }

}
